#include <stdarg.h>
#include "cp.h"
#include "cfpe.h"
#include "cf_pe.h"
#include "processConfig.h"
#include "debug.h"

extern int debug_level; /* see cp.c */

void Debug( int level, char *fmt, ... )
{
  va_list args;

  va_start(args, fmt);
  if ((level & debug_level) == level) {
      /* print out prefix of message */
    (void)fprintf(stderr, "Debug: ");
      /* print out remainder of message */
    (void)vfprintf(stderr, fmt, args);
    (void)fprintf(stderr, "\n");
  }
  va_end(args);
  fflush(stderr);
}

void Error( int level, char *fmt, ... )
{
  va_list args;

  va_start(args, fmt);
  if ((level & debug_level) == level) {
      /* print out prefix of message */
    (void)fprintf(stderr, "Error: ");
      /* print out remainder of message */
    (void)vfprintf(stderr, fmt, args);
    (void)fprintf(stderr, "\n");
  }
  va_end(args);
  fflush(stderr);
}
void Warning( int level, char *fmt, ... )
{
  va_list args;

  va_start(args, fmt);
  if ((level & debug_level) == level) {
      /* print out prefix of message */
    (void)fprintf(stderr, "Warni: ");
      /* print out remainder of message */
    (void)vfprintf(stderr, fmt, args);
    (void)fprintf(stderr, "\n");
  }
  va_end(args);
  fflush(stderr);
}

void Msg(int level, char *fmt, ... )
{
  va_list args;

  va_start(args, fmt);
  if ((level & debug_level) == level) {
      /* print out prefix of message 
    (void)fprintf(stdout, "Msg  : ");
       print out remainder of message */
    (void)vfprintf(stdout, fmt, args);
  }
  va_end(args);
  fflush(stdout);
}
void Info(int level, char *fmt, ... )
{
  va_list args;

  va_start(args, fmt);
  if ((level & debug_level) == level) {
      /* print out remainder of message */
    (void)vfprintf(stderr, fmt, args);
  }
  va_end(args);
  fflush(stderr);
}

int Show_rtst_pdu(int level, struct rtst_pdu_Type pdu)
{
 if ((level & debug_level) == level) 
  {
   Info(level, "Information about rtst_pdu:\n");
   Info(level, "rtst_pdu.type = %d\n", pdu.type);
   Info(level, "rtst_pdu.dstAddress = %d\n", pdu.dstAddress);
   Info(level, "rtst_pdu.usertitle[] = "); writeTitle(pdu.usertitle);
   Info(level, "rtst_pdu.confid[] = "); writeTitle(pdu.confid);
  } 
 return(0);
}

int Show_cdu(int level, struct cdu_Type cdu)
{
 Info(level, "Information about cdu:\n");
 switch (cdu.type) 
 {
  case JOIN:
  {
   struct join_cdu_Type *pcdu;
   pcdu = (struct join_cdu_Type *)&cdu;
   Info(level, "cdu.type = JOIN\n"); 
   Info(level, "cdu.usertitle[] = "); 
   writeTitle(pcdu->usertitle);
   Info(level, "cdu.confid[] = "); 
   writeTitle(pcdu->confid);
   break;
  }
  case DATAREQ:
  {
   struct datareq_cdu_Type *pcdu;
   pcdu = (struct datareq_cdu_Type *)&cdu;
   Info(level, "cdu.type = DATAREQ\n"); 
   Info(level, "cdu.length = %d\n", pcdu->length);
   Info(level, "cdu.data[] = "); 
   writeData(pcdu->data);

   break;
  }
  case DATAIND:
  {
   struct dataind_cdu_Type *pcdu;
   pcdu = (struct dataind_cdu_Type *)&cdu;
   Info(level, "cdu.type = DATAIND\n"); 
   Info(level, "cdu.usertitle[] = "); 
   writeTitle(pcdu->usertitle);
   Info(level, "cdu.length = %d\n", pcdu->length);
   Info(level, "cdu.data[] = "); 
   writeData(pcdu->data);
   break;
  }
  case LEAVE:
  {
   struct leave_cdu_Type *pcdu;
   pcdu = (struct leave_cdu_Type *)&cdu;
   Info(level, "cdu.type = LEAVE\n"); 
   break;
  }
  default:
  {
   Info(level, "Unknown cdu type!\n");
   break;
  }
 }
 return(0);
}

int Show_dataind_mdu(int level, struct dataind_mdu_Type mdu)
{
 Info(level, "Information about dataind_mdu:\n");
 Info(level, "mdu.srcAddress = %d\n",mdu.srcAddress);
 Info(level, "mdu.size       = %d\n",mdu.size);
 Info(level, "mdu.data       ="); 
 Show_pdu(level, mdu.data); 
 return(0);
}

int Show_datareq_mdu(int level, struct datareq_mdu_Type mdu)
{
 int i;
 Info(level, "Information about datareq_mdu:\n");
 i=0;
 while (mdu.dstAddress[i]!=0) {
 Info(level, "mdu.dstAddress[%d] = %d\n",i,mdu.dstAddress[i]);
 i++;
 }
 if (i==0) Info(level, "No md.dstAddresses!\n");
 Info(level, "mdu.data       ="); 
 Show_pdu(level, mdu.data);
 return(0);
}

int Show_pdu(int level, struct pdu_Type pdu)
{
 switch (pdu.type)
 {
  case JOIN:
  { 
   struct join_pdu_Type *ppdu;
   ppdu = (struct join_pdu_Type *)&pdu;
   Info(level, "Info about pdu...:\n");
   Info(level, "pdu.type = JOIN\n");
   Info(level, "pdu.usertitle[] = "); 
   writeTitle(ppdu->usertitle);
   Info(level, "pdu.confid[] = "); 
   writeTitle(ppdu->confid);
   break;
  }
  case LEAVE:
  {
   struct leave_pdu_Type *ppdu;
   ppdu = (struct leave_pdu_Type *)&pdu;
   Info(level, "pdu.type = LEAVE\n");
   Info(level, "pdu.usertitle[] = "); 
   writeTitle(ppdu->usertitle);
   Info(level, "pdu.confid[] = "); 
   writeTitle(ppdu->confid);
   break;
  }
  case ANSWER:
  {
   struct answer_pdu_Type *ppdu;
   ppdu = (struct answer_pdu_Type *)&pdu;
   Info(level, "pdu.type = ANSWER\n");
   Info(level, "pdu.usertitle[] = "); 
   writeTitle(ppdu->usertitle);
   Info(level, "pdu.confid[] = "); 
   writeTitle(ppdu->confid);
   break;
  }
  case DATA:
  {
   struct data_pdu_Type *ppdu;
   ppdu = (struct data_pdu_Type *)&pdu;
   Info(level, "pdu.type   = DATA\n");
   Info(level, "pdu.length = %d\n", ppdu->length);
   Info(level, "pdu.data[] = "); 
   writeData(ppdu->data);
   break;
  }
  default:
  {
   Info(level, "Unknown pdu!!!\n");
   break;
  }
 }
 return(0);
}
/******************************************************************/

int ShowConference(int level, struct conference_Type conference)
{
 Info(level, "active : %d\n",conference.active); 
 Info(level, "conference.confid :"); 
 writeTitle(conference.confid);
 Info(level, "conference.usertitle :");
 writeTitle(conference.usertitle);
 return(0);
}

/******************************************************************/

int ShowPartners(int level, struct partner_Type partner[MAXPARTNERS+1])
{
 int i;
 if (partner[0].msapid==0) 
 {
  Info(level,"No conference partners!\n");
  return(0);
 }
 for (i=0; partner[i].msapid!=0; i++)
 {
   Info(level,"msapid : %d, usertitle%d", partner[i].msapid,i); 
  writeTitle(partner[i].usertitle);
 }
 return(0);
}
int ShowConfig(int level,  struct ConfPartnersStruc *dest)
{
  int i;
  Info(level,"dest.size : %d \n",dest->size);
  for (i=0;i<dest->size;i++)
  {
    Info(level,"dest->Partner[%d].msapid = %d", \
	 i, dest->Partner[i].msapid);
   Info(level,"dest->Partner[%d].socketaddress{%s,%d}%s", i, \
	 inet_ntoa(dest->Partner[i].socketaddress.sin_addr), \
	 dest->Partner[i].socketaddress.sin_port, \
	 dest->Partner[i].hostname);
  }
  return(0);
}
