/******************************************************************/
/* filename: user2.c */
/* version : 1.00  */
/* author  : Jan Feenstra */
/******************************************************************/

#include "debug.h"
#include <sys/types.h>          /* gethostbyname */
#include <sys/socket.h>         /* gethostbyname */
#include <netinet/in.h>         /* gethostbyname */
#include <arpa/inet.h>          /* gethostbyname */
#include <netdb.h>              /* gethostbyname */
#include <stdio.h>              /* filestuff... */
#include <string.h>             /* strcpy(..) */
/******************************************************************/
/* External declaration of CSAP pipes [defined in cp.c] */
extern int CSAP_datareq_pipe[2]; /* We use only the [1] pipe for writing. */
extern int CSAP_dataind_pipe[2]; /* We use only the [0] pipe for reading. */

/************************************************************************/
#define BSIZE 1500

static int doread(int, int);

int user2()
{
	int sd;                 /* socket descriptor */
	fd_set read_mask;
	int read_result;
	
	Debug(Dbg,"CSAP main loop entered\n");
	for(;;) {
		/************* initialize buffers and time out ******************/

		FD_ZERO(&read_mask);
		FD_SET(CSAP_dataind_pipe[0], &read_mask);
		FD_SET(0, &read_mask);

		/******************** wait for an event to occur ***************/
		sd = select(32, &read_mask, (fd_set *) 0, (fd_set *) 0, 0);

		if(sd < 0) {
			Error(Dbg,"ERROR select\n");
			return(1);
		}
	
		if(FD_ISSET(CSAP_dataind_pipe[0], &read_mask)) {
			Debug(Dbg,"iut CSAP_dataind_pipe[0] selected\n");
			read_result = doread(CSAP_dataind_pipe[0], 1); /*  -> stdout */
			if (read_result != 0)
				return(read_result);
		} else if(FD_ISSET(0, &read_mask)) {
			Debug(Dbg,"iut stdin selected\n");
			read_result = doread(0, CSAP_datareq_pipe[1]); /* stdin ->  */
			if (read_result != 0)
				return(read_result);
		}
	}
}

static int doread(int from, int to)
{
	char *buffer[BSIZE];
	int nread = 0;
	int nwritten = 0;
	int write_result = 0;
	
	nread = read(from, (char *)&buffer, BSIZE);
	if (nread < 0) {
		Error(Dbg,"ERROR read\n");
		return(-1);
	}
	if (nread == 0) {
		Error(Dbg,"EOF read\n");
		return(1);
	}
	Debug(Dbg,"doread read %d\n", nread);
	while(nwritten < nread) {
		write_result = write(to, &buffer[nwritten], nread-nwritten);
		if (write_result >= 0) {
			nwritten += write_result;
		} else {
			Error(Dbg,"ERROR write\n");
			return(-1);
		}
	}
	return 0;
}
